from enum import Enum


class EmailTemplate(str, Enum):
    RESET_PASSWORD = "reset-password"
    NEW_COMMENT_ON_SUBMISSION = "new-comment-on-submission"
    SUBMISSION_ACCEPTED = "submission-accepted"
    NEW_SCHEDULE_INVITATION_ANSWER = "new-schedule-invitation-answer"

    def __str__(self) -> str:
        return str.__str__(self)



from dataclasses import dataclass
from datetime import datetime, timedelta, timezone
from enum import Enum
from typing import Any

import jwt


class Action(str, Enum):
    AUTH = "auth"

    def __str__(self) -> str:
        return str.__str__(self)


@dataclass
class PastaportoAction:
    action: Action
    payload: dict[str, Any]

    def sign(self, secret: str) -> str:
        now = datetime.now(timezone.utc)

        return jwt.encode(
            {
                "action": self.action,
                "payload": self.payload,
                "exp": now + timedelta(seconds=40),
                "iat": now,
            },
            str(secret),
            algorithm="HS256",
        )


def create_pastaporto_action(
    action: Action, payload: dict[str, str]
) -> PastaportoAction:
    return PastaportoAction(action=action, payload=payload)


def create_user_auth_pastaporto_action(
    user_id: int, jwt_auth_id: int
) -> PastaportoAction:
    return create_pastaporto_action(
        Action.AUTH, {"id": user_id, "jwtAuthId": jwt_auth_id}
    )



from typing import Any

import jwt
from pythonit_toolkit.headers import SERVICE_JWT_HEADER
from pythonit_toolkit.pastaporto.entities import Credential
from pythonit_toolkit.pastaporto.tokens import decode_service_to_service_token
from strawberry.permission import BasePermission
from strawberry.types import Info


class IsAuthenticated(BasePermission):
    message = "Not authenticated"

    def has_permission(self, source, info, **kwargs):
        return Credential.AUTHENTICATED in info.context.request.auth.scopes


def IsService(allowed_callers: list[str], secret: str, service: str):
    if not allowed_callers:
        raise ValueError("No callers allowed specified")

    if not secret:
        raise ValueError("JWT secret cannot be empty")

    if not service:
        raise ValueError("Current service name cannot be empty")

    secret = str(secret)

    class _IsService(BasePermission):
        message = "Forbidden"

        def has_permission(self, source: Any, info: Info, **kwargs) -> bool:
            token = info.context.request.headers.get(SERVICE_JWT_HEADER)

            for caller in allowed_callers:
                try:
                    decode_service_to_service_token(
                        token, secret, issuer=caller, audience=service
                    )
                    return True
                except (
                    jwt.DecodeError,
                    jwt.InvalidIssuerError,
                    jwt.ExpiredSignatureError,
                    jwt.InvalidAudienceError,
                ):
                    pass

            return False

    return _IsService



from dataclasses import dataclass
from typing import Any, Optional, Union

import httpx
from pythonit_toolkit.headers import SERVICE_JWT_HEADER
from pythonit_toolkit.pastaporto.tokens import generate_service_to_service_token


@dataclass
class ServiceResponse:
    data: Optional[dict[str, Any]]


@dataclass
class ServiceError(Exception):
    errors: list[dict[str, Any]]


class ServiceClient:
    def __init__(
        self,
        url: str,
        caller: str,
        service_name: str,
        jwt_secret: str,
    ):
        self.url = url
        self.caller = caller
        self.service_name = service_name
        self.jwt_secret = jwt_secret

        for arg in ("url", "caller", "service_name", "jwt_secret"):
            if not getattr(self, arg):
                raise ValueError(f"Argument '{arg}' can't be empty")

    async def execute(
        self,
        document: str,
        variables: Optional[dict[str, Any]] = None,
    ) -> Union[ServiceError, ServiceResponse]:
        token = generate_service_to_service_token(
            self.jwt_secret, issuer=self.caller, audience=self.service_name
        )

        async with httpx.AsyncClient(timeout=10) as client:
            response = await client.post(
                self.url,
                json={"query": document, "variables": variables},
                headers={
                    SERVICE_JWT_HEADER: token,
                },
            )

            response.raise_for_status()

            data = response.json()
            errors = data.get("errors", None)

            if errors:
                raise ServiceError(errors)

            return ServiceResponse(data=data.get("data"))
